@@ -4,7 +4,7 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '1.3001';
+our $VERSION = '1.4000';
 
 use Carp              ();
 use Fcntl             ();
@@ -30,7 +30,7 @@ our @EXPORT_OK = qw(
     html_builder
 );
 
-my $BYTECODE_VERSION = '1.4';
+my $BYTECODE_VERSION = '1.5';
 
 # $bytecode_version + $fullpath + $compiler_and_parser_options
 my $XSLATE_MAGIC   = qq{xslate;$BYTECODE_VERSION;%s;%s;};
@@ -64,6 +64,7 @@ use Text::Xslate::Util qw(
     dump
 );
 
+# constants
 BEGIN {
     our @ISA = qw(Exporter);
 
@@ -159,7 +160,8 @@ sub new {
 
     if($used != $nargs) {
         my @unknowns = grep { !exists $options->{$_} } keys %args;
-        warnings::warnif(misc => "$class: Unknown option(s): " . join ' ', @unknowns);
+        warnings::warnif(misc
+            => "$class: Unknown option(s): " . join ' ', @unknowns);
     }
 
     $args{path} = [
@@ -262,29 +264,33 @@ sub find_file {
     foreach my $p(@{$self->{path}}) {
         $self->note("  find_file: %s / %s ...\n", $p, $file) if _DUMP_LOAD;
 
-        my $path_id;
+        my $cache_prefix;
         if(ref $p eq 'HASH') { # virtual path
             defined(my $content = $p->{$file}) or next;
-            $fullpath   = \$content;
-            # TODO: we should provide a way to specify this mtime
-            #       (like as Template::Provider?)
-            $orig_mtime = $^T;
-            $path_id    = 'HASH';
+            $fullpath = \$content;
+
+            # NOTE:
+            # Because contents of virtual paths include their digest,
+            # time-dependent cache verifier makes no sense.
+            $orig_mtime   = 0;
+            $cache_mtime  = 0;
+            $cache_prefix = 'HASH';
         }
         else {
             $fullpath = File::Spec->catfile($p, $file);
             defined($orig_mtime = (stat($fullpath))[_ST_MTIME])
                 or next;
-            $path_id    = Text::Xslate::uri_escape($p);
+            $cache_prefix = Text::Xslate::uri_escape($p);
         }
 
         # $file is found
         $cachepath = File::Spec->catfile(
             $self->{cache_dir},
-            $path_id,
+            $cache_prefix,
             $file . 'c',
         );
-        $cache_mtime = (stat($cachepath))[_ST_MTIME]; # may fail, but doesn't matter
+        # stat() will be failed if the cache doesn't exist
+        $cache_mtime = (stat($cachepath))[_ST_MTIME];
         last;
     }
 
@@ -498,16 +504,21 @@ sub _magic_token {
     ]);
 
     if(ref $fullpath) { # ref to content string
-        require 'Digest/MD5.pm';
-        my $md5 = Digest::MD5->new();
-        my $s = ${$fullpath};
-        utf8::encode($s);
-        $md5->add($s);
-        $fullpath = join ':', ref($fullpath), $md5->hexdigest();
+        $fullpath = join ':', ref($fullpath),
+            $self->_digest(${$fullpath});
     }
     return sprintf $XSLATE_MAGIC, $fullpath, $self->{serial_opt};
 }
 
+sub _digest {
+    my($self, $content) = @_;
+    require 'Digest/MD5.pm'; # we don't want to create its namespace
+    my $md5 = Digest::MD5->new();
+    utf8::encode($content);
+    $md5->add($content);
+    return $md5->hexdigest();
+}
+
 sub _extract_options {
     my($self, $opt_ref) = @_;
     my @options;
@@ -548,7 +559,8 @@ sub _compiler {
 
 sub compile {
     my $self = shift;
-    return $self->_compiler->compile(@_, from_include => $self->{from_include});
+    return $self->_compiler->compile(@_,
+        from_include => $self->{from_include});
 }
 
 sub _error {
@@ -570,7 +582,7 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 1.3001.
+This document describes Text::Xslate version 1.4000.
 
 =head1 SYNOPSIS
 
@@ -631,38 +643,22 @@ compiled into intermediate code, and then executed by the virtual machine,
 which is highly optimized for rendering templates. Thus, Xslate is
 much faster than any other template engines.
 
-Here is a result of F<benchmark/x-rich-env.pl> to compare various template
-engines in I<rich> environment where applications are persistent and XS modules
-are available.
+The template roundup project by Sam Graham shows Text::Xslate got very
+high scores in I<instance_reuse> condition (i.e. for persistent application).
 
-    $ perl -Mblib benchmark/x-rich-env.pl
-    Perl/5.10.1 i686-linux
-    Text::Xslate/0.2002
-    Text::MicroTemplate/0.18
-    Text::MicroTemplate::Extended/0.11
-    Template/2.22
-    Text::ClearSilver/0.10.5.4
-    HTML::Template::Pro/0.9503
-    1..4
-    ok 1 - TT: Template-Toolkit
-    ok 2 - MT: Text::MicroTemplate
-    ok 3 - TCS: Text::ClearSilver
-    ok 4 - HTP: HTML::Template::Pro
-    Benchmarks with 'include' (datasize=100)
-              Rate     TT     MT    TCS    HTP Xslate
-    TT       129/s     --   -84%   -94%   -95%   -99%
-    MT       807/s   527%     --   -63%   -71%   -96%
-    TCS     2162/s  1580%   168%     --   -23%   -89%
-    HTP     2814/s  2087%   249%    30%     --   -85%
-    Xslate 19321/s 14912%  2295%   794%   587%     --
+=over
 
-According to this result, Xslate is 100+ times faster than Template-Toolkit.
-Text::MicroTemplate is a very fast template engine written in pure Perl, but
-XS-based modules, namely Text::ClearSilver, HTML::Template::Pro and Xslate
-are faster than Text::MicroTemplate. Moreover, Xslate is even faster than
-Text::ClearSilver and HTML::Template::Pro.
+=item The template roundup project
 
-There are benchmark scripts in the F<benchmark/> directory.
+L<http://illusori.co.uk/projects/Template-Roundup/>
+
+=item Perl Template Roundup October 2010 Performance vs Variant Report: instance_reuse
+
+L<http://illusori.co.uk/projects/Template-Roundup/201010/performance_vs_variant_by_feature_for_instance_reuse.html>
+
+=back
+
+There are some benchmarks in F<benchmark/> directory in the Xslate distribution.
 
 =head3 Smart escaping for HTML meta characters
 
@@ -995,6 +991,14 @@ C<< %% ... >> line code, instead of C<< <: ... :> >> and C<< : ... >>.
 B<TTerse> is a syntax that is a subset of Template-Toolkit 2 (and partially TT3),
 which is explained in L<Text::Xslate::Syntax::TTerse>.
 
+=item HTMLTemplate
+
+There's HTML::Template compatible layers in CPAN.
+
+L<Text::Xslate::Syntax::HTMLTemplate> is a syntax for HTML::Template.
+
+L<HTML::Template::Parser> is a converter from HTML::Template to Text::Xslate.
+
 =back
 
 =head1 NOTES
@@ -1070,7 +1074,7 @@ the same in internals:
 
 =item *
 
-Customization of the default escaping filter
+An "too-safe" HTML escaping filter which escape all the symbolic characters
 
 =back
 
@@ -1080,6 +1084,8 @@ Customization of the default escaping filter
 
 WEB: L<http://xslate.org/>
 
+ML: L<http://groups.google.com/group/xslate>
+
 IRC: #xslate @ irc.perl.org
 
 REPOSITORY:
